/*Copyright (c) 2019 Alexis Nasr && Franck Dary

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:i

 The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/
/// \file BaseConfig.hpp
/// \brief Configuration of the ReadingMachine.
/// \author Franck Dary
/// @version 2.0
/// @date 2019-12-10

#ifndef BASECONFIG__H
#define BASECONFIG__H

#include <string>
#include <vector>
#include <unordered_map>
#include <boost/flyweight.hpp>
#include "util.hpp"
#include "Config.hpp"

class SubConfig;

class BaseConfig : public Config
{
  private :

  std::vector<std::string> colIndex2Name;
  std::unordered_map<std::string, int> colName2Index;

  Utf8String rawInputUtf8;

  private :

  void readMCD(std::string_view mcdFilename);
  void readRawInput(std::string_view rawFilename);
  void readTSVInput(std::string_view tsvFilename);

  public :

  BaseConfig(std::string_view mcdFilename, std::string_view tsvFilename, std::string_view rawFilename);

  std::size_t getNbColumns() const override;
  std::size_t getFirstLineIndex() const override;
  std::size_t getColIndex(const std::string & colName) const override;
  bool hasColIndex(const std::string & colName) const override;
  const std::string & getColName(int colIndex) const override;

  friend SubConfig;
};

#endif
