#include "DepthLayerTreeEmbedding.hpp"

DepthLayerTreeEmbeddingImpl::DepthLayerTreeEmbeddingImpl(std::vector<int> maxElemPerDepth, int embeddingsSize, int outEmbeddingsSize, std::vector<std::string> columns, std::vector<int> focusedBuffer, std::vector<int> focusedStack, LSTMImpl::LSTMOptions options) :
  maxElemPerDepth(maxElemPerDepth), columns(columns), focusedBuffer(focusedBuffer), focusedStack(focusedStack)
{
  for (unsigned int i = 0; i < maxElemPerDepth.size(); i++)
    depthLstm.emplace_back(register_module(fmt::format("lstm_{}",i), LSTM(columns.size()*embeddingsSize, outEmbeddingsSize, options)));
}

torch::Tensor DepthLayerTreeEmbeddingImpl::forward(torch::Tensor input)
{
  auto context = input.narrow(1, firstInputIndex, getInputSize());

  std::vector<torch::Tensor> outputs;

  int offset = 0;
  for (unsigned int focused = 0; focused < focusedBuffer.size()+focusedStack.size(); focused++)
    for (unsigned int depth = 0; depth < maxElemPerDepth.size(); depth++)
    {
      outputs.emplace_back(depthLstm[depth](context.narrow(1, offset, maxElemPerDepth[depth]*columns.size()).view({input.size(0), maxElemPerDepth[depth], (long)columns.size()*input.size(2)})));
      offset += maxElemPerDepth[depth]*columns.size();
    }

  return torch::cat(outputs, 1);
}

std::size_t DepthLayerTreeEmbeddingImpl::getOutputSize()
{
  std::size_t outputSize = 0;

  for (unsigned int depth = 0; depth < maxElemPerDepth.size(); depth++)
    outputSize += depthLstm[depth]->getOutputSize(maxElemPerDepth[depth]);

  return outputSize*(focusedBuffer.size()+focusedStack.size());
}

std::size_t DepthLayerTreeEmbeddingImpl::getInputSize()
{
  int inputSize = 0;
  for (int maxElem : maxElemPerDepth)
    inputSize += (focusedBuffer.size()+focusedStack.size())*maxElem*columns.size();
  return inputSize;
}

void DepthLayerTreeEmbeddingImpl::addToContext(std::vector<std::vector<long>> & context, Dict & dict, const Config & config, bool) const
{
  std::vector<long> focusedIndexes;

  for (int index : focusedBuffer)
    focusedIndexes.emplace_back(config.getRelativeWordIndex(index));

  for (int index : focusedStack)
    if (config.hasStack(index))
      focusedIndexes.emplace_back(config.getStack(index));
    else
      focusedIndexes.emplace_back(-1);

  for (auto & contextElement : context)
    for (auto index : focusedIndexes)
    {
      std::vector<std::string> childs{std::to_string(index)};

      for (unsigned int depth = 0; depth < maxElemPerDepth.size(); depth++)
      {
        std::vector<std::string> newChilds;
        for (auto & child : childs)
          if (config.has(Config::childsColName, std::stoi(child), 0))
          {
            auto val = util::split(config.getAsFeature(Config::childsColName, std::stoi(child)).get(), '|');
            newChilds.insert(newChilds.end(), val.begin(), val.end());
          }
        childs = newChilds;

        for (int i = 0; i < maxElemPerDepth[depth]; i++)
          for (auto & col : columns)
            if (i < (int)newChilds.size() and config.has(col, std::stoi(newChilds[i]), 0))
              contextElement.emplace_back(dict.getIndexOrInsert(config.getAsFeature(col,std::stoi(newChilds[i]))));
            else
              contextElement.emplace_back(dict.getIndexOrInsert(Dict::nullValueStr));
      }
    }
}

