#include "FocusedColumnLSTM.hpp"

FocusedColumnLSTMImpl::FocusedColumnLSTMImpl(std::vector<int> focusedBuffer, std::vector<int> focusedStack, std::string column, int maxNbElements, int embeddingsSize, int outEmbeddingsSize, LSTMImpl::LSTMOptions options) : focusedBuffer(focusedBuffer), focusedStack(focusedStack), column(column), maxNbElements(maxNbElements)
{
  lstm = register_module("lstm", LSTM(embeddingsSize, outEmbeddingsSize, options));
}

torch::Tensor FocusedColumnLSTMImpl::forward(torch::Tensor input)
{
  std::vector<torch::Tensor> outputs;
  for (unsigned int i = 0; i < focusedBuffer.size()+focusedStack.size(); i++)
    outputs.emplace_back(lstm(input.narrow(1, firstInputIndex+i*maxNbElements, maxNbElements)));

  return torch::cat(outputs, 1);
}

std::size_t FocusedColumnLSTMImpl::getOutputSize()
{
  return (focusedBuffer.size()+focusedStack.size())*lstm->getOutputSize(maxNbElements);
}

std::size_t FocusedColumnLSTMImpl::getInputSize()
{
  return (focusedBuffer.size()+focusedStack.size()) * maxNbElements;
}

void FocusedColumnLSTMImpl::addToContext(std::vector<std::vector<long>> & context, Dict & dict, const Config & config, bool) const
{
  std::vector<long> focusedIndexes;

  for (int index : focusedBuffer)
    focusedIndexes.emplace_back(config.getRelativeWordIndex(index));

  for (int index : focusedStack)
    if (config.hasStack(index))
      focusedIndexes.emplace_back(config.getStack(index));
    else
      focusedIndexes.emplace_back(-1);

  for (auto & contextElement : context)
  {
    for (auto index : focusedIndexes)
    {
      if (index == -1)
      {
        for (int i = 0; i < maxNbElements; i++)
          contextElement.emplace_back(dict.getIndexOrInsert(Dict::nullValueStr));
        continue;
      }

      std::vector<std::string> elements;
      if (column == "FORM")
      {
        auto asUtf8 = util::splitAsUtf8(config.getAsFeature(column, index).get());

        for (int i = 0; i < maxNbElements; i++)
          if (i < (int)asUtf8.size())
            elements.emplace_back(fmt::format("{}", asUtf8[i]));
          else
            elements.emplace_back(Dict::nullValueStr);
      }
      else if (column == "FEATS")
      {
        auto splited = util::split(config.getAsFeature(column, index).get(), '|');

        for (int i = 0; i < maxNbElements; i++)
          if (i < (int)splited.size())
            elements.emplace_back(fmt::format("FEATS({})", splited[i]));
          else
            elements.emplace_back(Dict::nullValueStr);
      }
      else if (column == "ID")
      {
        if (config.isTokenPredicted(index))
          elements.emplace_back("ID(TOKEN)");
        else if (config.isMultiwordPredicted(index))
          elements.emplace_back("ID(MULTIWORD)");
        else if (config.isEmptyNodePredicted(index))
          elements.emplace_back("ID(EMPTYNODE)");
      }
      else if (column == "EOS")
      {
        bool isEOS = config.getAsFeature(Config::EOSColName, index) == Config::EOSSymbol1;
        elements.emplace_back(fmt::format("EOS({})", isEOS));
      }
      else
      {
        elements.emplace_back(config.getAsFeature(column, index));
      }

      if ((int)elements.size() != maxNbElements)
        util::myThrow(fmt::format("elements.size ({}) != maxNbElements ({})", elements.size(), maxNbElements));

      for (auto & element : elements)
        contextElement.emplace_back(dict.getIndexOrInsert(element));
    }
  }
}

