#include "RawInputLSTM.hpp"

RawInputLSTMImpl::RawInputLSTMImpl(int leftWindow, int rightWindow, int embeddingsSize, int outEmbeddingsSize, LSTMImpl::LSTMOptions options) : leftWindow(leftWindow), rightWindow(rightWindow)
{
  lstm = register_module("lstm", LSTM(embeddingsSize, outEmbeddingsSize, options));
}

torch::Tensor RawInputLSTMImpl::forward(torch::Tensor input)
{
  return lstm(input.narrow(1, firstInputIndex, getInputSize()));
}

std::size_t RawInputLSTMImpl::getOutputSize()
{
  return lstm->getOutputSize(leftWindow + rightWindow + 1);
}

std::size_t RawInputLSTMImpl::getInputSize()
{
  return leftWindow + rightWindow + 1;
}

void RawInputLSTMImpl::addToContext(std::vector<std::vector<long>> & context, Dict & dict, const Config & config, bool) const
{
  if (leftWindow < 0 or rightWindow < 0)
    return;

  for (auto & contextElement : context)
  {
    for (int i = 0; i < leftWindow; i++)
      if (config.hasCharacter(config.getCharacterIndex()-leftWindow+i))
        contextElement.push_back(dict.getIndexOrInsert(fmt::format("{}", config.getLetter(config.getCharacterIndex()-leftWindow+i))));
      else
        contextElement.push_back(dict.getIndexOrInsert(Dict::nullValueStr));

    for (int i = 0; i <= rightWindow; i++)
      if (config.hasCharacter(config.getCharacterIndex()+i))
        contextElement.push_back(dict.getIndexOrInsert(fmt::format("{}", config.getLetter(config.getCharacterIndex()+i))));
      else
        contextElement.push_back(dict.getIndexOrInsert(Dict::nullValueStr));
  }
}

